<CFCOMPONENT HINT="This component is used to manage an application using some undocumented features.">
	<cffunction name="killApplication" returntype="void" output="false">
		<cfargument name="appName" type="string" required="true">
		<cfscript>
		var oApp = createObject("java","coldfusion.runtime.ApplicationScopeTracker");
		var myApp = oApp.getApplicationScope(arguments.appName);
		oApp.cleanUp(myApp);
		</cfscript>
	</cffunction>

	<CFFUNCTION NAME="setAppVariable" RETURNTYPE="void" OUTPUT="false" HINT="Sets an application variable on ANY application on the current machine.">
		<CFARGUMENT NAME="sAppName" TYPE="string" REQUIRED="true" HINT="The name of the application on which you want to set a variable.">
		<CFARGUMENT NAME="sVarName" TYPE="string" REQUIRED="true" HINT="The name of the variable you want to set.">
		<CFARGUMENT NAME="sValue" TYPE="any" REQUIRED="true" HINT="The value of the variable you want to set.">
	
		<CFSCRIPT>
			var bAppFound = false;
			var enumApps = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationKeys();		
			var oApp = "";
		
			//Make sure the application exists before we try to do anything
			for ( ; enumApps.hasMoreElements(); ) {
				if (enumApps.nextElement() IS arguments.sAppName) {
					bAppFound = true;
					break;
				};
			}
		
			//if the applicaiton exists, now we can get a reference to it and set a variable.
			if (bAppFound) {
				oApp = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationScope(arguments.sAppName);
				SetVariable("oApp.#arguments.sVarName#", arguments.sValue);
			}
		</CFSCRIPT>
		
		<CFIF NOT bAppFound>
			<CFTHROW TYPE="Application" MESSAGE="Application not found." DETAIL="The application named '#arguments.sAppName#' does not exist on this CF server.">
		</CFIF>
	</CFFUNCTION>

	<CFFUNCTION NAME="callAppFunc" RETURNTYPE="any" OUTPUT="false">
		<CFARGUMENT NAME="sAppName" TYPE="string" REQUIRED="true" HINT="The name of the application on which you want to set a variable.">
		<CFARGUMENT NAME="sFuncName" TYPE="string" REQUIRED="true">
		<CFARGUMENT NAME="sVarName" TYPE="string" REQUIRED="false" HINT="The name of the variable you want to set.">
		<CFARGUMENT NAME="sValue" TYPE="any" REQUIRED="false" HINT="The value of the variable you want to set.">
	
		<cfset var bAppFound = false>
		<cfset var enumApps = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationKeys()>
		<cfset var oApp = "">
		<!--- Make sure the application exists before we try to do anything --->
		<cfloop condition="enumApps.hasMoreElements()">
			<cfif (enumApps.nextElement() IS arguments.sAppName)>
				<cfset bAppFound = true>
				<cfbreak>
			</cfif>
		</cfloop>
		
		<!--- if the applicaiton exists, now we can get a reference to it and set a variable. --->
		<cfif (bAppFound)>
			<cfset oApp = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationScope(arguments.sAppName)>
			<cfif IsDefined("arguments.sVarName")>
				<cfreturn Evaluate("oApp.#arguments.sFuncName#(arguments.sVarName, arguments.sValue)")>
			<cfelse>
				<cfreturn Evaluate("oApp.#arguments.sFuncName#()")>
			</cfif>
		</cfif>
		
		<CFIF NOT bAppFound>
			<CFTHROW TYPE="Application" MESSAGE="Application not found." DETAIL="The application named '#arguments.sAppName#' does not exist on this CF server.">
		</CFIF>
	</CFFUNCTION>
	
	
	<CFFUNCTION NAME="getAppVariable" RETURNTYPE="any" OUTPUT="false" HINT="Gets an application variable From ANY application on the current machine.">
		<CFARGUMENT NAME="sAppName" TYPE="string" REQUIRED="true" HINT="The name of the application From which you want to retrieve a variable.">
		<CFARGUMENT NAME="sVarName" TYPE="string" REQUIRED="true" HINT="The name of the variable you want to get.">
	
		<CFSCRIPT>
			var bAppFound = false;
			var bVarFound = false;
			var enumApps = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationKeys();		
			var oApp = "";
		
			//Make sure the application exists before we try to do anything
			for ( ; enumApps.hasMoreElements(); ) {
				if (enumApps.nextElement() IS arguments.sAppName) {
					bAppFound = true;
					break;
				}
			}
		
			//if the applicaiton exists, now we can get a reference to it and set a variable.
			if (bAppFound) {
				oApp = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationScope(arguments.sAppName);
				bVarFound = StructKeyExists(oApp, arguments.sVarName);
	
				if (bVarFound) {
					return Evaluate("oApp.#arguments.sVarName#");
				}
			}
		</CFSCRIPT>
		
		<CFIF NOT bAppFound>
			<CFTHROW TYPE="Application" MESSAGE="Application not found." DETAIL="The application named '#arguments.sAppName#' does not exist on this CF server.">
		<CFELSEIF NOT bVarFound>
			<CFTHROW TYPE="Application" MESSAGE="Variable not found." DETAIL="The variable name 'application.#arguments.sVarName#' does not exist in the application named '#arguments.sAppName#'.">
		</CFIF>
	</CFFUNCTION>
	
	
	<CFFUNCTION NAME="getAllAppsQuery" RETURNTYPE="query" OUTPUT="false" HINT="Returns a query containing all application names on the current server.">
		<CFSCRIPT>
			var qryApps = QueryNew("appname");
			var enumApps = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationKeys();		
		
			//Make sure the application exists before we try to do anything
			for ( ; enumApps.hasMoreElements(); ) {
				QueryAddRow(qryApps);
				QuerySetCell(qryApps, "appname", enumApps.nextElement());
			}
			
			return qryApps;
		</CFSCRIPT>
	</CFFUNCTION>
	
	
	<CFFUNCTION NAME="getAllAppsStruct" RETURNTYPE="struct" OUTPUT="false" HINT="Returns a complex structure containing all applications and app variables on the current server.">
		<CFSCRIPT>
			var stApps = StructNew();
			var enumApps = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationKeys();		
			var sAppName = "";
		
			//Make sure the application exists before we try to do anything
			for ( ; enumApps.hasMoreElements(); ) {
				sAppName = enumApps.nextElement();
				stApps[sAppName] = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationScope(sAppName);
			}
			
			return stApps;
		</CFSCRIPT>
	</CFFUNCTION>
	
	
	<CFFUNCTION NAME="getApp" RETURNTYPE="struct" OUTPUT="false" HINT="Gets ANY application on the current machine.">
		<CFARGUMENT NAME="sAppName" TYPE="string" REQUIRED="true" HINT="The name of the application you want to retrieve.">
	
		<CFSCRIPT>
			var bAppFound = false;
			var enumApps = createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationKeys();		
		
			//Make sure the application exists before we try to do anything
			for ( ; enumApps.hasMoreElements(); ) {
				if (enumApps.nextElement() IS arguments.sAppName) {
					bAppFound = true;
					break;
				}
			}
		
			//if the applicaiton exists, now we can get a reference to it and set a variable.
			if (bAppFound) {
				return createObject("java", "coldfusion.runtime.ApplicationScopeTracker").getApplicationScope(arguments.sAppName);
			}
		</CFSCRIPT>
		
		<CFIF NOT bAppFound>
			<CFTHROW TYPE="Application" MESSAGE="Application not found." DETAIL="The application named '#arguments.sAppName#' does not exist on this CF server.">
		</CFIF>
	</CFFUNCTION>
</CFCOMPONENT>