<!--- 
ehCacheManager - CF Interface to EHCache.  
Created By:  Andrew Powell
Created On:  11/1/07

Method Summary:

cacheInit(cachename,scope) - initializes the cachemanager and loads the XML config for the appropriate scope (fully customizeable per scope)
cacheSetElement(key, val)         - adds something to the cache with a given key.  if the key already exists, an Update of the key is done.
cacheGetElement(key)              - retrieves the given key From the cache.  if the cache is empty or has expired, returns an empty string.
cacheRemoveElement(key)           - removes a given key From the cache.
cacheShutdown()            - flushes and shuts the cachemanager down
getCurrentCache()     - convenience method to get the current cache repository
getCurrentCacheKeys() - returns an array of the current key names in the cache
findCache(cachname)   - checks to see if a given cache name exists in the cache manager
getInternals()        - debugging method that returns the variables scope of the cfc
--->

<cfcomponent output="false">
	<cffunction name="cacheInit" output="No" returntype="any">
		<!--- This creates an instance of the cache manager in the application scope --->
		<cfreturn createObject("component", "com.caching.universalmind.ehCache.ehCacheManager").init(cacheName="applicationCache", scope="com/caching/universalmind/ehCache/application")>
	</cffunction>

	<cffunction name="init" returntype="ehCacheManager" access="public" output="false">
		<cfargument name="cacheName" type="string" required="true"/>
		<cfargument name="scope" type="string" required="true"/>
		<cfscript>
			var mBeanServer     = createObject('java','java.lang.management.ManagementFactory').getPlatformMBeanServer();
			
			variables.manager 	= createObject('java','net.sf.ehcache.CacheManager').init(javacast("string",expandPath('#arguments.scope#.xml')));
			variables.cacheName = arguments.cacheName;
			
			createObject('java','net.sf.ehcache.management.ManagementService').registerMBeans(variables.manager,mBeanServer,javacast("boolean",false),javacast("boolean",false),javacast("boolean",false),javacast("boolean",true));
		
			variables.manager.addCache(javacast("string",arguments.cacheName));

			return this;
		</cfscript>
	</cffunction>

	<cffunction name="cacheSetElement" returntype="boolean" access="public" output="false">
		<cfargument name="key" type="string" required="true"/>
		<cfargument name="val" type="any"    required="true"/>
		<cfscript>
			var element = createObject('java','net.sf.ehcache.Element').init(javacast("string",arguments.key),arguments.val);
			var cache   = getCurrentCache();
			try{
				cache.put(element);
				return true;
			}
			catch(e Any){
				return false;
			}
		</cfscript>
	</cffunction>
	
	<cffunction name="cacheGetElement" returntype="any" access="public" output="false">
		<cfargument name="key" type="string" required="true"/>
		<cfscript>
			var cache 	= getCurrentCache();
			var element = cache.get(javacast("string",arguments.key));
			if(isDefined("element")){
				return element.getValue();
			}
			else{
				return "";
			}
		</cfscript>
	</cffunction>
	
	<cffunction name="cacheRemoveElement" returntype="void" access="public" output="false">
		<cfargument name="key" type="string" required="true"/>
		<cfscript>
			var cache 	= getCurrentCache();
			cache.remove(javacast("string",arguments.key));
		</cfscript>
	</cffunction>
	
	<cffunction name="cacheShutdown" returntype="void" access="public" output="false">
		<cfset variables.manager.shutdown()/>
	</cffunction>
	
	<cffunction name="getCurrentCache" returntype="any" access="public" output="false">
		<cfreturn variables.manager.getCache(javacast("string",variables.cacheName))/>
	</cffunction>
  
  <cffunction name="getCurrentCacheKeys" returntype="any" access="public" output="false">
		<cfreturn variables.manager.getCache(javacast("string",variables.cacheName)).getKeys()/>
	</cffunction>
	
	<cffunction name="findCache" returntype="boolean" access="public" output="false">
		<cfargument name="cacheTest" type="string" required="true"/>
		<cfscript>
			var cacheNames   = variables.manager.getCacheNames();
			var cacheNameLen = arrayLen(cacheNames);
			var i            = 1;
			var cacheCheck   = false;
			
			for(i=1;i LTE cacheNameLen;i=i+1)
			{
				if(arguments.cacheTest EQ cacheNames[i])
				{
					cacheCheck = true;
					break;
				}	
			}
			return cacheCheck;
		</cfscript>
	</cffunction>
	
	<cffunction name="getInternals" returntype="struct" access="public" output="false">
		<cfreturn variables/>
	</cffunction>
</cfcomponent>